/*
 * Spritebend by Anders Montonen
 *
 * This program is released into the public domain
 * without restrictions.
 *
 * Controls:
 *  X - select vertex 1
 *  Y - select vertex 2
 *  B - select vertex 3
 *  A - select vertex 4
 *  D-pad - move selected vertex
 *
 */

#include "types.h"
#include "vdp1.h"
#include "vdp2.h"
#include "smpc.h"

#include "sprites.h"

typedef u32 padstate_t;

#define Pad_LS  (1<<12)
#define Pad_S   (1<<11)
#define Pad_A   (1<<10)
#define Pad_C   (1<<9)
#define Pad_B   (1<<8)
#define Pad_R   (1<<7)
#define Pad_L   (1<<6)
#define Pad_D   (1<<5)
#define Pad_U   (1<<4)
#define Pad_RS  (1<<3)
#define Pad_X   (1<<2)
#define Pad_Y   (1<<1)
#define Pad_Z   (1<<0)
/* > */
typedef struct
{
    s16 x;
    s16 y;
} coord_t;

coord_t coords[4] =
{
    { 80, 56 },
    { 240, 56 },
    { 240, 168 },
    { 80, 168 }
};

int pad_poll(padstate_t *pad);

int main(void)
{
    drawing_command_t *dcptr = (drawing_command_t *)VDP1_VRAM;
    u16 *palptr = (u16*)(VDP1_VRAM_TOP - 32);
    u16 *spr2ptr = (u16*)(VDP1_VRAM_TOP - 64);
    u16 *spr1ptr = (u16*)(VDP1_VRAM_TOP - 2112);
    int vertex = 0;     /* which vertex are we controlling now? */
    int i;
    padstate_t pad = 0;
    int xdir = 0;
    int ydir = 0;
    
    /* init pad */
    PDR1 = 0;
    DDR1 = 0x60;
    IOSEL = IOSEL1;
    
    WaitForVBLANK();
    /* init VDP2 */
    TVMD = 0;
    EXTEN = 0;
    BGON = 0;
    /* Enable RGB-mode sprite data */
    SPCTL = SPCLMD;
    /* Set sprites to highest priority */
    PRISA = 0x707;
    PRISB = 0x707;
    PRISC = 0x707;
    PRISD = 0x707;
    
    /* init VDP1 */
    FBCR = 0;
    TVMR = TVMNormal;
    PTMR = PlotAuto;
    EWDR = 0;
    EWLR = 0;
    EWRR = ((640/8)<<8)|223;
    
    /* write sprite 1 to vram */
    for (i = 0; i < 1024; i++)
    {
        spr1ptr[i] = checkerboard[i];
    }
    /* write sprite 2 to vram */
    for (i = 0; i < 16; i++)
    {
        spr2ptr[i] = marker[i];
    }
    /* write palette to vram */
    for (i = 0; i < 16; i++)
    {
        palptr[i] = palette[i];
    }
    
    /* write the initial tables to vram */
    /* set system clipping coords */
    dcptr[0].CMDCTRL = Ctrl_SYSTEMCLIP|Ctrl_JUMPNEXT;
    dcptr[0].CMDXC = 320;
    dcptr[0].CMDYC = 224;
    /* set local coordinate set */
    dcptr[1].CMDCTRL = Ctrl_LOCALCOORD|Ctrl_JUMPNEXT;
    dcptr[1].CMDXA = 0;
    dcptr[1].CMDYA = 0;
    /* draw sprite */
    dcptr[2].CMDCTRL = Ctrl_DIST_SPRITE|Ctrl_JUMPNEXT;
    dcptr[2].CMDPMOD = Pmod_ECD|Pmod_CM1;
    dcptr[2].CMDCOLR = (int)palptr>>3;
    dcptr[2].CMDSRCA = (int)spr1ptr>>3;
    dcptr[2].CMDSIZE = ((64/8)<<8)|64;
    dcptr[2].CMDXA = coords[0].x;
    dcptr[2].CMDYA = coords[0].y;
    dcptr[2].CMDXB = coords[1].x;
    dcptr[2].CMDYB = coords[1].y;
    dcptr[2].CMDXC = coords[2].x;
    dcptr[2].CMDYC = coords[2].y;
    dcptr[2].CMDXD = coords[3].x;
    dcptr[2].CMDYD = coords[3].y;
    /* draw polyline */
    dcptr[3].CMDCTRL = Ctrl_JUMPNEXT|Ctrl_POLYLINE;
    dcptr[3].CMDPMOD = Pmod_ECD|Pmod_SPD;
    dcptr[3].CMDCOLR = (1<<15)|(0xf<<5)|0xf;
    dcptr[3].CMDXA = coords[0].x;
    dcptr[3].CMDYA = coords[0].y;
    dcptr[3].CMDXB = coords[1].x;
    dcptr[3].CMDYB = coords[1].y;
    dcptr[3].CMDXC = coords[2].x;
    dcptr[3].CMDYC = coords[2].y;
    dcptr[3].CMDXD = coords[3].x;
    dcptr[3].CMDYD = coords[3].y;
    /* draw/skip indicator sprite */
    dcptr[4].CMDCTRL = Ctrl_SKIPNEXT|Ctrl_SPRITE;
    dcptr[4].CMDPMOD = Pmod_ECD|Pmod_CML16;
    dcptr[4].CMDCOLR = (int)palptr>>3;
    dcptr[4].CMDSRCA = (int)spr2ptr>>3;
    dcptr[4].CMDSIZE = (1<<8)|8;
    dcptr[4].CMDXA = coords[vertex-1].x-4;
    dcptr[4].CMDYA = coords[vertex-1].y-4;
    /* stop drawing */
    dcptr[5].CMDCTRL = Ctrl_END;

    /* switch on screen */
    TVMD = DISP|NOINTERLACE|VRES224|HRES320;

    while(1)
    {
        WaitForVBLANKExit();
        WaitForVBLANK();
        pollpad(&pad);
        /* check which vertex is selected, if any */
        if (pad & (Pad_X|Pad_Y|Pad_A|Pad_B))
        {
            /* only one vertex can be selected */
            switch (vertex)
            {
                case 0:
                case 1:
                    if (pad & Pad_X)
                        vertex = 1;
                    else if (pad & Pad_Y)
                        vertex = 2;
                    else if (pad & Pad_B)
                        vertex = 3;
                    else
                        vertex = 4;
                    break;
                case 2:
                    if (pad & Pad_Y)
                        break;
                    else if (pad & Pad_X)
                        vertex = 1;
                    else if (pad & Pad_B)
                        vertex = 3;
                    else
                        vertex = 4;
                    break;
                case 3:
                    if (pad & Pad_B)
                        break;
                    else if (pad & Pad_X)
                        vertex = 1;
                    else if (pad & Pad_Y)
                        vertex = 2;
                    else
                        vertex = 4;
                    break;
                case 4:
                    if (pad & Pad_A)
                        break;
                    else if (pad & Pad_X)
                        vertex = 1;
                    else if (pad & Pad_Y)
                        vertex = 2;
                    else
                        vertex = 3;
                    break;
            }
            ydir = xdir = 0;
            if (pad & 0x10)
                ydir = -1;
            if (pad & 0x20)
                ydir = 1;
            if (pad & 0x40)
                xdir = -1;
            if (pad & 0x80)
                xdir = 1;
            coords[vertex-1].x += xdir;
            if (coords[vertex-1].x < 0)
                coords[vertex-1].x = 0;
            if (coords[vertex-1].x > 320)
                coords[vertex-1].x = 320;
            coords[vertex-1].y += ydir;
            if (coords[vertex-1].y < 0)
                coords[vertex-1].y = 0;
            if (coords[vertex-1].y > 224)
                coords[vertex-1].y = 224;
            dcptr[2].CMDXA = coords[0].x;
            dcptr[2].CMDYA = coords[0].y;
            dcptr[2].CMDXB = coords[1].x;
            dcptr[2].CMDYB = coords[1].y;
            dcptr[2].CMDXC = coords[2].x;
            dcptr[2].CMDYC = coords[2].y;
            dcptr[2].CMDXD = coords[3].x;
            dcptr[2].CMDYD = coords[3].y;
            dcptr[3].CMDXA = coords[0].x;
            dcptr[3].CMDYA = coords[0].y;
            dcptr[3].CMDXB = coords[1].x;
            dcptr[3].CMDYB = coords[1].y;
            dcptr[3].CMDXC = coords[2].x;
            dcptr[3].CMDYC = coords[2].y;
            dcptr[3].CMDXD = coords[3].x;
            dcptr[3].CMDYD = coords[3].y;
            dcptr[4].CMDXA = coords[vertex-1].x-4;
            dcptr[4].CMDYA = coords[vertex-1].y-4;
            dcptr[4].CMDCTRL = Ctrl_JUMPNEXT|Ctrl_SPRITE;
        }
        else
        {
            vertex = 0;
            dcptr[4].CMDCTRL = Ctrl_SKIPNEXT|Ctrl_SPRITE;
        }
    }
    return 0;
}

int pollpad(padstate_t *pad)
{
    padstate_t temp;
    
    PDR1 = 0x60;
    temp = (PDR1&0x8)<<9;
    PDR1 = 0x40;
    temp |= (PDR1&0xf)<<8;
    PDR1 = 0x20;
    temp |= (PDR1&0xf)<<4;
    PDR1 = 0x00;
    temp |= (PDR1&0xf);
    
    temp ^= 0x1fff;
    
    *pad = temp;
    
    return 0;
}
